/** @file indexed.h
 *
 *  Interface to GiNaC's index carrying objects. */

/*
 *  GiNaC Copyright (C) 1999-2001 Johannes Gutenberg University Mainz, Germany
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GINAC_INDEXED_H__
#define __GINAC_INDEXED_H__

#include <string>
#include "exprseq.h"

namespace GiNaC {


/** Base class for objects with indices. */
class indexed : public exprseq
{
	GINAC_DECLARE_REGISTERED_CLASS(indexed, exprseq)

	// other constructors
public:
	indexed(const ex & i1);
	indexed(const ex & i1, const ex & i2);
	indexed(const ex & i1, const ex & i2, const ex & i3);
	indexed(const ex & i1, const ex & i2, const ex & i3, const ex & i4);
	indexed(const exvector & iv);
	indexed(exvector * iv);

	// functions overriding virtual functions from base classes
public:
	void printraw(std::ostream & os) const;
	void printtree(std::ostream & os, unsigned indent) const;
	void print(std::ostream & os, unsigned upper_precedence=0) const;
	void printcsrc(std::ostream & os, unsigned type, unsigned upper_precedence) const;
	bool info(unsigned inf) const;

	/** Return the vector of indices on this object. */
	exvector get_indices(void) const {return seq;}

protected:
	ex derivative(const symbol & s) const;
	bool is_equal_same_type(const basic & other) const;
	unsigned return_type(void) const;
	unsigned return_type_tinfo(void) const;
	ex thisexprseq(const exvector & v) const;
	ex thisexprseq(exvector * vp) const;

	// new virtual functions which can be overridden by derived classes
	// none
	
	// non-virtual functions in this class
protected:
	void printrawindices(std::ostream & os) const;
	void printtreeindices(std::ostream & os, unsigned indent) const;
	void printindices(std::ostream & os) const;
	bool all_of_type_idx(void) const;

// member variables
	// none
};

// utility functions
inline const indexed &ex_to_indexed(const ex &e)
{
	return static_cast<const indexed &>(*e.bp);
}

} // namespace GiNaC

#endif // ndef __GINAC_INDEXED_H__
